<%
'(P) 2003-05, C.K

'------------------------------------------------------------------------------
'Blog 帖子实体及相关操作类
'需要事先包含 CommonFunctions.asp 和 DataOperator 及 EnDec 类的定义！
'------------------------------------------------------------------------------

'Blog 帖子的信息
'UPD@050605: 增加 AuthorID, FromIP 字段的映射
'Updated @ 060811: 增加 Tags 字段的映射
Class BlogInfo
	Private m_blogId
	Private m_belongTo
	Private m_postTime
	Private m_title
	Private m_authorId
	Private m_authorName
	Private m_expression
	Private m_content
	Private m_fromIP
	Private m_categoryId
	Private m_categoryName
	Private m_allowComment
	Private m_state
	Private m_viewCount
	Private m_commentNum
	Private m_tags

	Private Sub Class_Initialize()
		m_blogId = 0
		m_belongTo = 0
		m_postTime = Null
		m_title = ""
		m_authorId = 0
		m_authorName = ""
		m_expression = "1"
		m_content = ""
		m_categoryId = 1
		m_categoryName = "未分类"
		m_allowComment = True
		m_state = 1 '0：保密；1：正常；2：草稿
		m_viewCount = 0
		m_commentNum = 0
		m_tags = ""
	End Sub

	'Blog 帖子 ID
	Public Property Get BlogID()
		BlogID = m_blogId
	End Property
	Public Property Let BlogID(v_blogId)
		m_blogId = v_blogId
	End Property

	'帖子归属
	Public Property Get BelongTo()
		BelongTo = m_belongTo
	End Property
	Public Property Let BelongTo(v_belongTo)
		m_belongTo = v_belongTo
	End Property

	'发表时间
	Public Property Get PostTime()
		PostTime = m_postTime
	End Property
	Public Property Let PostTime(v_postTime)
		m_postTime = v_postTime
	End Property

	'标题
	Public Property Get Title()
		Title = m_title
	End Property
	Public Property Let Title(v_title)
		m_title = v_title
	End Property

	'作者 ID
	Public Property Get AuthorID()
		AuthorID = m_authorId
	End Property
	Public Property Let AuthorID(v_authorId)
		m_authorId = v_authorId
	End Property

	'作者名
	Public Property Get AuthorName()
		AuthorName = m_authorName
	End Property
	Public Property Let AuthorName(v_authorName)
		m_authorName = v_authorName
	End Property

	'表情图标代码
	Public Property Get Expression()
		Expression = m_expression
	End Property
	Public Property Let Expression(v_expression)
		m_expression = v_expression
	End Property

	'内容
	Public Property Get Content()
		Content = m_content
	End Property
	Public Property Let Content(v_content)
		m_content = v_content
	End Property

	'来自 IP
	Public Property Get FromIP()
		FromIP = m_fromIP
	End Property
	Public Property Let FromIP(v_fromIP)
		m_fromIP = v_fromIP
	End Property

	'分类 ID
	Public Property Get CategoryID()
		CategoryID = m_categoryId
	End Property
	Public Property Let CategoryID(v_categoryId)
		m_categoryId = v_categoryId
	End Property

	'分类名
	Public Property Get CategoryName()
		CategoryName = m_categoryName
	End Property
	Public Property Let CategoryName(v_categoryName)
		m_categoryName = v_categoryName
	End Property

	'是否允许评论？
	Public Property Get AllowComment()
		AllowComment = m_allowComment
	End Property
	Public Property Let AllowComment(v_allowComment)
		m_allowComment = v_allowComment
	End Property

	'帖子状态
	Public Property Get PostState()
		PostState = m_state
	End Property
	Public Property Let PostState(v_state)
		m_state = v_state
	End Property

	'点击数
	Public Property Get ViewCount()
		ViewCount = m_viewCount
	End Property
	Public Property Let ViewCount(v_viewCount)
		m_viewCount = v_viewCount
	End Property

	'评论数
	Public Property Get CommentNum()
		CommentNum = m_commentNum
	End Property
	Public Property Let CommentNum(v_commentNum)
		m_commentNum = v_commentNum
	End Property

	'Tags 文本串（空格分隔）
	Public Property Get Tags()
		Tags = m_tags
	End Property
	Public Property Let Tags(v_tags)
		m_tags = v_tags
	End Property
End Class

'Blog 文章操作类
Class BlogsDB
	Private m_connString
	Private m_dataOperator
	Private m_oneBlogInfo
	Private m_currentBloggerId

	Private Sub Class_Initialize()
		m_connString = ""
		m_currentBloggerId = 0
		Set m_oneBlogInfo = New BlogInfo
		Set m_dataOperator = New DataOperator
	End Sub

	Private Sub Class_Terminate()
		Set m_dataOperator = Nothing
		Set m_oneBlogInfo = Nothing
	End Sub

	Public Property Let DataConnectionString(connStr)
		m_connString = connStr
		m_dataOperator.OpenDatabase connStr
	End Property

	Public Property Let CurrentBloggerID(bloggerId)
		m_currentBloggerId = bloggerId
	End Property

	Private Function GetCategoryNameByID(cateId)
		Dim retval
		If cateId = 1 Then '系统最初默认的分类项目
			retval = "未分类"
			GetCategoryNameByID = retval
		Else
			retval = m_dataOperator.ExecuteArray("SELECT [CateName] FROM T_Categories WHERE [BelongTo] = @belongTo AND [CateID] = @cateId", Array(m_currentBloggerId, cateId))
			If Not IsArray(retval) Then
				Err.Raise vbObjectError + 1000, "nirvana: BlogsDB", "Specified category does not exist!"
			End If
			GetCategoryNameByID = CStr(retval(0, 0))
		End If
	End Function

	'添加网志
	Public Sub AddBlogEntry(entry)
		Dim title, authorId, authorName, expression, content, fromIP, categoryId, allowComment, postState, tags
		title = entry.Title : authorId = entry.AuthorID : authorName = entry.AuthorName
		expression = entry.Expression : content = entry.Content : fromIP = entry.FromIP
		categoryId = entry.CategoryID : allowComment = entry.AllowComment : postState = entry.PostState
		tags = entry.Tags

		Dim categoryName
		categoryName = GetCategoryNameByID(categoryId)
		m_dataOperator.ExecuteNonQuery "INSERT INTO T_Blogs ([BelongTo], [Title], [AuthorID], [AuthorName], [Expression], [Content], [FromIP], [CategoryID], [CategoryName], [AllowComment], [State], [Tags]) VALUES (@belongTo, @title, @authorId, @authorName, @expression, @content, @fromIP, @categoryId, @categoryName, @allowComment, @state, @tags)", Array(m_currentBloggerId, title, authorId, authorName, expression, content, fromIP, categoryId, categoryName, allowComment, postState, tags)
		If postState < 2 Then '如果发表的不是草稿，则更新网志计数
			m_dataOperator.ExecuteNonQuery "UPDATE T_Bloggers SET [BlogNum] = [BlogNum] + 1 WHERE [UID] = @userId", Array(m_currentBloggerId)
			m_dataOperator.ExecuteNonQuery "UPDATE T_Bloggers SET [LastUpdateTime] = ? WHERE [UID] = @userId", Array(Now(), m_currentBloggerId)
		End If
		'清除有关缓存……
	End Sub

	'重新设置 Blog 的最后更新时间
	Private Sub SetBloggerLastUpdateTime()
		Dim arrLastUpdateTime
		arrLastUpdateTime = m_dataOperator.ExecuteArray("SELECT TOP 1 [PostTime] FROM T_Blogs WHERE [BelongTo] = @belongTo AND [State] < 2 ORDER BY [BID] DESC", Array(m_currentBloggerId))
		If Not IsNull(arrLastUpdateTime) Then
			m_dataOperator.ExecuteNonQuery "UPDATE T_Bloggers SET [LastUpdateTime] = @lastUpdateTime WHERE [UID] = @userId", Array(arrLastUpdateTime(0, 0), m_currentBloggerId)
		Else
			m_dataOperator.ExecuteNonQuery "UPDATE T_Bloggers SET [LastUpdateTime] = @lastUpdateTime WHERE [UID] = @userId", Array(Null, m_currentBloggerId)
		End If
	End Sub

	'更新网志信息
	Public Sub UpdateBlogEntry(entry)
		Dim blogId, title, expression, content, categoryId, allowComment, postState, tags
		blogId = entry.BlogID : title = entry.Title : expression = entry.Expression
		content = entry.Content : categoryId = entry.CategoryID : allowComment = entry.AllowComment : postState = entry.PostState
		tags = entry.Tags

		Dim categoryName
		categoryName = GetCategoryNameByID(categoryId)
		m_dataOperator.ExecuteNonQuery "UPDATE T_Blogs SET [Title] = @title, [Expression] = @expression, [Content] = @content, [CategoryID] = @categoryId, [CategoryName] = @categoryName, [AllowComment] = @allowComment, [State] = @state, [Tags] = @tags WHERE [BID] = @blogId AND [BelongTo] = @belongTo", Array(title, expression, content, categoryId, categoryName, allowComment, postState, tags, blogId, m_currentBloggerId)
		'如果被修改为草稿相关，则更新 Blog 的网志计数
		RefreshBlogNum m_currentBloggerId
		'重新设置 Blog 的最后更新时间：Copied from DeleteBlogEntry()
		SetBloggerLastUpdateTime
		'清除有关缓存……
	End Sub

	'更新网志计数，因为设置状态为“草稿”会导致发贴量变化
	Private Sub RefreshBlogNum(bloggerId)
		Dim cnt
		cnt = m_dataOperator.ExecuteArray("SELECT COUNT([BID]) FROM T_Blogs WHERE [BelongTo] = @userId AND [State] < 2", Array(bloggerId))
		m_dataOperator.ExecuteNonQuery "UPDATE T_Bloggers SET [BlogNum] = @newNum WHERE [UID] = @userId", Array(CLng(cnt(0, 0)), bloggerId)
	End Sub

	'删除一条网志
	Public Sub DeleteBlogEntry(blogId)
		m_dataOperator.ExecuteNonQuery "DELETE FROM T_Blogs WHERE [BID] = @blogId AND [BelongTo] = @belongTo", Array(blogId, m_currentBloggerId)
		m_dataOperator.ExecuteNonQuery "DELETE FROM T_Tags WHERE [EntryID] = @entryId AND [BloggerID] = @bloggerId", Array(blogId, m_currentBloggerId) '删除这篇文章的 tags
		RefreshBlogNum m_currentBloggerId
		m_dataOperator.ExecuteNonQuery "DELETE FROM T_Comments WHERE [CommentTo] = @commentTo", Array(blogId) '删除帖子后面的评论
		'重新设置 Blog 的最后更新时间：Copied from DeleteBlogEntry()
		SetBloggerLastUpdateTime
		'清除有关缓存……
	End Sub

	Public Sub BatchDeleteEntries(blogIds)
		m_dataOperator.ExecuteNonQuery "DELETE FROM T_Blogs WHERE [BID] IN (" & blogIds & ") AND [BelongTo] = @belongTo", Array(m_currentBloggerId)
		m_dataOperator.ExecuteNonQuery "DELETE FROM T_Tags WHERE [EntryID] IN (" & blogIds & ") AND [BloggerID] = @bloggerId", Array(m_currentBloggerId) '删除相关的 tags
		RefreshBlogNum m_currentBloggerId
		m_dataOperator.ExecuteNonQuery "DELETE FROM T_Comments WHERE [CommentTo] IN (" & blogIds & ")", Null '删除帖子后面的评论
		'重新设置 Blog 的最后更新时间
		SetBloggerLastUpdateTime
		'清除有关缓存……
	End Sub

	'获得一篇网志的信息
	Public Function GetBlogEntry(blogId)
		Dim blogEntryInfo
		blogEntryInfo = m_dataOperator.ExecuteArray("SELECT [BID], [BelongTo], [PostTime], [Title], [AuthorName], [Expression], [Content], [CategoryID], [CategoryName], [AllowComment], [State], [ViewCount], [CommentNum], [Tags] FROM T_Blogs WHERE [BID] = @blogId", Array(blogId))
		If IsNull(blogEntryInfo) Then
			Err.Raise vbObjectError + 1000, "nirvana: BlogsDB", "Specified blog entry does not exist!"
		End If
		If CInt(blogEntryInfo(1, 0)) <> m_currentBloggerId Then
			Err.Raise vbObjectError + 1000, "nirvana: BlogsDB", "Specified entry does not belong to the corresponding Blog!"
		End If

		m_oneBlogInfo.BlogID = blogEntryInfo(0, 0)
		m_oneBlogInfo.BelongTo = blogEntryInfo(1, 0)
		m_oneBlogInfo.PostTime = blogEntryInfo(2, 0)
		m_oneBlogInfo.Title = blogEntryInfo(3, 0)
		m_oneBlogInfo.AuthorName = blogEntryInfo(4, 0)
		m_oneBlogInfo.Expression = blogEntryInfo(5, 0)
		m_oneBlogInfo.Content = blogEntryInfo(6, 0)
		m_oneBlogInfo.CategoryID = blogEntryInfo(7, 0)
		m_oneBlogInfo.CategoryName = blogEntryInfo(8, 0)
		m_oneBlogInfo.AllowComment = blogEntryInfo(9, 0)
		m_oneBlogInfo.PostState = blogEntryInfo(10, 0)
		m_oneBlogInfo.ViewCount = blogEntryInfo(11, 0)
		m_oneBlogInfo.CommentNum = blogEntryInfo(12, 0)
		m_oneBlogInfo.Tags = blogEntryInfo(13, 0)

		Set GetBlogEntry = m_oneBlogInfo
	End Function

	'获得某篇 blog 的评论列表（经过认证的）
	Public Function GetBlogComments(blogId)
		Dim retval
		retval = m_dataOperator.ExecuteArray("SELECT [CID], [CommentTo], [AuthorName], [EMail], [PostTime], [Content], [FromIP], [Link] FROM T_Comments WHERE [CommentTo] = @commentTo AND [State] = 1 ORDER BY [CID]", Array(blogId))
		GetBlogComments = retval
	End Function

	'网志归类
	Public Sub CategorizeEntries(blogIds, cateId)
		Dim cateName
		cateName = GetCategoryNameByID(cateId)
		m_dataOperator.ExecuteNonQuery "UPDATE T_Blogs SET [CategoryID] = @cateId, [CategoryName] = @cateName WHERE [BelongTo] = @belongTo AND [BID] IN (" & blogIds & ")", Array(cateId, cateName, m_currentBloggerId)
		'清除有关缓存
	End Sub

	'改变网志状态
	Public Sub SetStateOfEntries(blogIds, newState)
		m_dataOperator.ExecuteNonQuery "UPDATE T_Blogs SET [State] = @state WHERE [BelongTo] = @belongTo AND [BID] IN (" & blogIds & ")", Array(newState, m_currentBloggerId)
		RefreshBlogNum m_currentBloggerId
		'重新设置 Blog 的最后更新时间：Copied from DeleteBlogEntry()
		SetBloggerLastUpdateTime
		'清除有关缓存
	End Sub

	'批量设置 Tag
	Public Sub BatchSetTags(blogIds, tagString)
		m_dataOperator.ExecuteNonQuery "UPDATE T_Blogs SET [Tags] = @tags WHERE [BelongTo] = @belongTo AND [BID] IN (" & blogIds & ")", Array(tagString, m_currentBloggerId)
		'清除有关缓存
	End Sub

	'网志查看计数器（暂时屏蔽）
	Public Sub AddViewCount(blogId)
		m_dataOperator.ExecuteNonQuery "UPDATE T_Blogs SET [ViewCount] = [ViewCount] + 1 WHERE [BID] = @blogId", Array(blogId)
	End Sub

	Public Function GetViewCount(blogId)
		Dim retval
		retval = m_dataOperator.ExecuteArray("SELECT [ViewCount] FROM T_Blogs WHERE [BID] = @blogId", Array(blogId))
		GetViewCount = retval(0, 0)
	End Function

	'获取最新一篇网志的 ID，用于发表网志时同步添加 Tag 信息
	Public Function GetLatestEntryID()
		Dim rv
		rv = m_dataOperator.ExecuteArray("SELECT TOP 1 [BID] FROM T_Blogs WHERE BelongTo = @bloggerId ORDER BY [BID] DESC", Array(m_currentBloggerId))
		If IsNull(rv) Then
			GetLatestEntryID = 0
		Else
			GetLatestEntryID = CLng(rv(0, 0))
		End If
	End Function

	'获得更新一篇帖子的链接文字（公开网志）
	Public Function GetPrevEntryLink(blogId)
		Dim rv
		rv = m_dataOperator.ExecuteArray("SELECT TOP 1 [BID], [Title] FROM T_Blogs WHERE [BelongTo] = @bloggerId AND [BID] > @entryId AND [State] = 1 ORDER BY [BID]", Array(m_currentBloggerId, blogId))
		If IsNull(rv) Then
			GetPrevEntryLink = "没有更新的网志了"
		Else
			GetPrevEntryLink = "<a href='Read.asp?ID=" & CStr(m_currentBloggerId) & "&amp;BID=" & rv(0, 0) & "'>" & Server.HTMLEncode(rv(1, 0)) & "</a>"
		End If
	End Function

	'获得更老一篇帖子的链接文字（公开网志）
	Public Function GetNextEntryLink(blogId)
		Dim rv
		rv = m_dataOperator.ExecuteArray("SELECT TOP 1 [BID], [Title] FROM T_Blogs WHERE [BelongTo] = @bloggerId AND [BID] < @entryId AND [State] = 1 ORDER BY [BID] DESC", Array(m_currentBloggerId, blogId))
		If IsNull(rv) Then
			GetNextEntryLink = "没有更老的网志了"
		Else
			GetNextEntryLink = "<a href='Read.asp?ID=" & CStr(m_currentBloggerId) & "&amp;BID=" & rv(0, 0) & "'>" & Server.HTMLEncode(rv(1, 0)) & "</a>"
		End If
	End Function
End Class
%>
